#include <gint/display.h>
#include <gint/keyboard.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define rgb(r, g, b) ((r & 0xf8) << 8) | ((g & 0xfc) << 3) | ((b & 0xf8) >> 3)

unsigned int color[256];

extern int width=0;
extern int height=0;
extern int nbFrames=0;
extern char* name="";

unsigned char** frame;

void ReadFrameVideo(FILE* fp){
    for (int i = 0; i < height; i++)
        for (int j = 0; j < width; j++)
        {
            unsigned char c;
            fread(&c, 1, 1, fp);
            frame[i][j] = c;
        }
}

void DrawFrameVideo(){
    int decal = (396-(width*2))/4;
    for (int i = 0; i < height; i++)
        for (int j = 0; j < width; j++)
            drect((j+ decal)*2, i*2, (j+ decal)*2+2, i*2+2, color[frame[i][j]]);
}



int IsKeyDownWait(int key){
    if (keydown(key)){
        while (keydown(key)){
            clearevents();
        }
        return 1;
    }
    return 0;
}

extern bopti_image_t bottomUI;
extern bopti_image_t topUI;
extern bopti_image_t midUI;

void DrawUILoad(){
    for (int i = 16; i < 396; i+=12)
    {
        dclear(C_BLACK);
        dimage(0,195,&bottomUI);
        dimage(0,0,&topUI);
        dimage(160,69,&midUI);
        drect(i,200,i+20,205,rgb(0,255,0));
        dtext((396/2)-((13*8)/2),180,C_WHITE,name);
        clearevents();
        dupdate();
    }
}

void LoadInfo(){
    FILE* fp = fopen("VIDEO.bin", "rb");
    fread(&width, 1, sizeof(int), fp);
    fread(&height, 1, sizeof(int), fp);
    fread(&nbFrames, 1, sizeof(int), fp);
    int nameSize;
    fread(&nameSize, 1, sizeof(int), fp);
    name = malloc(nameSize+1);
    frame = malloc(sizeof(unsigned char*)*height);
    for (int i = 0; i < height; i++)
        frame[i] = malloc(sizeof(unsigned char)*width);
    for (int i = 0; i < nameSize; i++)
    {
        fread(&name[i], 1, 1, fp);
    }
    name[nameSize] = '\0';
    fclose(fp);
}


void Video(){
    while(1)
    {
        FILE* fp = fopen("VIDEO.bin", "rb");
        int width, height, nbFrames;
        fread(&width, 1, sizeof(int), fp);
        fread(&height, 1, sizeof(int), fp);
        fread(&nbFrames, 1, sizeof(int), fp);
        int nameSize;
        fread(&nameSize, 1, sizeof(int), fp);
        char tmp;
        for (int i = 0; i < nameSize; i++)
        {
            fread(&tmp, 1, 1, fp);
        }
        for (int i = 0; i < nbFrames; i++)
        {
            dclear(C_BLACK);
            gint_world_switch(GINT_CALL(ReadFrameVideo,(void*)fp));
            DrawFrameVideo();
            clearevents();
            if (keydown(KEY_MENU))
            {
                    fclose(fp);
                    return;
            }
            dupdate();
        }
        fclose(fp);
    }
}

int VideoExists(){
    FILE* fp = fopen("VIDEO.bin", "rb");
    if (fp == NULL)
        return 0;
    fclose(fp);
    return 1;
}

void DrawUIError(){
    dclear(C_BLACK);
    dimage(0,195,&bottomUI);
    dimage(0,0,&topUI);
    dimage(160,69,&midUI);
    dtext((396/2)-((26*8)/2),110,C_WHITE,"Error: VIDEO.bin not found");
    clearevents();
    dupdate();
}

int main(void)
{
    for (int i = 0; i < 256; i++)
        color[i] = rgb(i, i, i);
    if (!VideoExists()){
        DrawUIError();
        while(1)
        {
            clearevents();
            if (IsKeyDownWait(KEY_MENU))
                return 1;
        }
    }
    LoadInfo();
    for(int i = 0; i < 5; i++)
        DrawUILoad();
    Video();
    return 1;
}

